import json
import sys

from my_clippings.classes.clipping import Clipping
from my_clippings.utils.reader import read_clippings_file_into_array


def clippings_to_json(my_clippings_file: str):
    """ """
    books = {}

    for raw_clipping in read_clippings_file_into_array(my_clippings_file):
        try:
            parsed_clipping = Clipping(raw_clipping).parsed
            current_book = parsed_clipping.get("title")

            if current_book not in books:
                books[current_book] = [parsed_clipping]
            else:
                books[current_book].append(parsed_clipping)

        except Exception as e:
            print(f"An error occurred: {e}")

    return books


if __name__ == "__main__":
    print("hello")
    print(sys.argv[1])
    file_name = sys.argv[1] if len(sys.argv) > 1 else "My Clippings.txt"
    books = clippings_to_json(file_name)

    with open("books.json", "w") as fp:
        json.dump(books, fp)



import ftfy

SEPARATOR = "=========="
SECTIONS = {0: "title_and_author", 1: "metadata", 2: "content"}


def read_clippings_file_into_array(my_clippings_file):
    """Pre-process clippings file into array of clippings"""
    clippings = [{}]

    with open(my_clippings_file, "r", encoding="utf-8") as clippings_file:
        curr_line = 0
        index = 0

        for line in clippings_file:
            # If separator, start new clipping
            if line.strip() == SEPARATOR:
                curr_line = 0
                index += 1
                clippings.append({})
            # Skip empty lines
            elif line.strip() == "":
                continue
            # Clipping lines
            else:
                section = SECTIONS.get(curr_line, "Invalid")
                line = ftfy.fixes.remove_control_chars(line).strip()
                clippings[index][section] = line
                curr_line += 1

    # Remove empty clippings
    filtered_clippings = [clipping for clipping in clippings if clipping]

    return filtered_clippings



import re

import dateparser

TRANSLATIONS = {
    "note_type": {
        "note": ["note", "nota", "的笔记"],
        "highlight": ["highlight", "subrayado"],
        "bookmark": ["bookmark", "marcador"],
        "dividers": ["页"],
    },
    "location": ["Location", "posición"],
}

title_author_regex = re.compile(
    r"(?P<title>.+)(?:\(|\[|\-)(?P<author>(?<=\(|\[|\-).+?(?=\)|\]|$))"
)
position_regex = re.compile(r"(\d+)-?(\d+)?")


def parse_title_and_author(title_and_author: str):
    """Parse title and author from the raw clipping."""
    result = title_author_regex.findall(title_and_author)
    if result:
        title, author = result[0]
        title = title.strip()
        author = author.strip()
        return title, author
    else:
        for word in TRANSLATIONS["location"]:
            if word in title_and_author.lower():
                return "", ""

        return title_and_author, ""


def parse_metadata(metadata: str):
    """Parse metadata from the raw clipping."""
    result = metadata.split("|")

    note_type = parse_note_type(result[0])
    location = parse_location(result[1]) if len(result) == 3 else None
    date = parse_date(result[2]) if len(result) == 3 else parse_date(result[1])

    return note_type, location, date


def parse_location(raw_location: str):
    """Parse location into a tuple."""
    try:
        result = position_regex.findall(raw_location)
        if result:
            start, end = result[0]
            start = int(start)
            end = int(end) if end else None
            return (start, end) if start and end else (start)
    except Exception:
        return None


def parse_date(raw_date: str):
    """Parse date into a tuple."""
    try:
        result = raw_date.replace(",", "").split(" ")

        # Remove two letter words from the list that are not numbers
        result = [word for word in result if len(word) > 2 or word.isdigit()]

        month = result[2]
        day = result[3]
        year = result[4]

        date = day + " " + month + " " + year
        date = dateparser.parse(date).strftime("%Y-%m-%d")
        return date
    except Exception:
        return None


def parse_note_type(raw_note_type: str):
    """Parse note type into a string."""
    try:
        for key, value in TRANSLATIONS["note_type"].items():
            if any(word in raw_note_type.lower() for word in value):
                return key
    except Exception:
        return None



import pytest

from my_clippings.utils.parsers import (
    parse_date,
    parse_metadata,
    parse_note_type,
    parse_title_and_author,
)
from tests.test_cases import (
    date_tests,
    metadata_tests,
    note_type_tests,
    title_and_author_tests,
)


@pytest.mark.parametrize("testcase", title_and_author_tests)
def test_parse_title_and_author(testcase):
    input_data, expected_title, expected_author = testcase

    title, author = parse_title_and_author(input_data)

    assert (
        title == expected_title
    ), f"Expected title '{expected_title}', but got '{title}'. Input: '{input_data}'"

    assert (
        author == expected_author
    ), f"Expected author '{expected_author}', but got '{author}'. Input: '{input_data}'"


@pytest.mark.parametrize("testcase", metadata_tests)
def test_metadata(testcase):
    input_data, expected_note_type, expected_location, expected_date = testcase

    note_type, location, date = parse_metadata(input_data)

    assert (
        note_type == expected_note_type
    ), f"Expected note_type '{expected_note_type}', but got '{note_type}'. Input: '{input_data}'"

    assert (
        location == expected_location
    ), f"Expected location '{expected_location}', but got '{location}'. Input: '{input_data}'"

    assert (
        date == expected_date
    ), f"Expected date '{expected_date}', but got '{date}'. Input: '{input_data}'"


@pytest.mark.parametrize("testcase", date_tests)
def test_date(testcase):
    input_data, expected_date = testcase

    date = parse_date(input_data)

    assert (
        date == expected_date
    ), f"Expected date '{expected_date}', but got '{date}'. Input: '{input_data}'"


@pytest.mark.parametrize("testcase", note_type_tests)
def test_note_type(testcase):
    input_data, expected_note_type = testcase

    note_type = parse_note_type(input_data)

    assert (
        note_type == expected_note_type
    ), f"Expected note_type '{expected_note_type}', but got '{note_type}'. Input: '{input_data}'"



from collections import namedtuple

TitleAndAuthorTestCase = namedtuple("TestCase", "input title author")
title_and_author_tests = [
    TitleAndAuthorTestCase(
        "The Ultimate Hitchhiker's Guide to the Galaxy: Five Novels in One Outrageous Volume",
        "The Ultimate Hitchhiker's Guide to the Galaxy: Five Novels in One Outrageous Volume",
        "",
    ),
    TitleAndAuthorTestCase(
        "Meditations (Marcus Aurelius)", "Meditations", "Marcus Aurelius"
    ),
    TitleAndAuthorTestCase(
        "To Kill A Mockingbird (Harper Lee)", "To Kill A Mockingbird", "Harper Lee"
    ),
    TitleAndAuthorTestCase(
        "A Mind For Numbers (Barbara Oakley)", "A Mind For Numbers", "Barbara Oakley"
    ),
    TitleAndAuthorTestCase(
        "The Autobiography of Benjamin Franklin (AmazonClassics Edition) (Franklin, Benjamin)",
        "The Autobiography of Benjamin Franklin (AmazonClassics Edition)",
        "Franklin, Benjamin",
    ),
    TitleAndAuthorTestCase(
        "Altered Traits (Daniel Goleman;Richard Davidson)",
        "Altered Traits",
        "Daniel Goleman;Richard Davidson",
    ),
    TitleAndAuthorTestCase(
        "Madhouse at the End of the Earth: The Belgica's Journey into the Dark Antarctic Night (Julian Sancton)",
        "Madhouse at the End of the Earth: The Belgica's Journey into the Dark Antarctic Night",
        "Julian Sancton",
    ),
    TitleAndAuthorTestCase(
        "Joel on Software: And on Diverse and Occasionally Related Matters That Will Prove of Interest to Software Developers, Designers, and Managers, and to Those Who, Whether by Good Fortune or Ill Luck, Work with Them in Some Capacity (Joel Spolsky)",
        "Joel on Software: And on Diverse and Occasionally Related Matters That Will Prove of Interest to Software Developers, Designers, and Managers, and to Those Who, Whether by Good Fortune or Ill Luck, Work with Them in Some Capacity",
        "Joel Spolsky",
    ),
    TitleAndAuthorTestCase(
        "Alex's Adventures in Numberland (Bellos, Alex)",
        "Alex's Adventures in Numberland",
        "Bellos, Alex",
    ),
    TitleAndAuthorTestCase(
        "Code Complete, Second Edition eBook - Steve McConnell",
        "Code Complete, Second Edition eBook",
        "Steve McConnell",
    ),
    TitleAndAuthorTestCase("", "", ""),
]

MetadataTestCase = namedtuple("TestCase", "input note_type location date")
metadata_tests = [
    MetadataTestCase(
        "Your Highlight on page 31 | Location 447-448 | Added on Saturday, January 6, 2024",
        "highlight",
        (447, 448),
        "2024-01-06",
    ),
    MetadataTestCase(
        "Your Highlight on page 47 | Location 723 | Added on Sunday, January 7, 2024 10:25:48 AM",
        "highlight",
        (723),
        "2024-01-07",
    ),
    MetadataTestCase(
        "Your Highlight on page 107 | Location 1709-1711 | Added on Wednesday, January 10, 2024 1:20:25 PM",
        "highlight",
        (1709, 1711),
        "2024-01-10",
    ),
    MetadataTestCase(
        "Your Highlight on page 107 | Added on Wednesday, January 10, 2024 1:20:25 PM",
        "highlight",
        None,
        "2024-01-10",
    ),
    MetadataTestCase(
        "La subrayado en la página 236 | posición 3614-3615 | Añadido el lunes, 10 de julio de 2023 13:23:06",
        "highlight",
        (3614, 3615),
        "2023-07-10",
    ),
    MetadataTestCase(
        "La subrayado en la página 236 | posición 3614-3615 | Añadido el lunes, 10 de julio de 2023 13:23:06",
        "highlight",
        (3614, 3615),
        "2023-07-10",
    ),
]

DateTestCase = namedtuple("TestCase", "input date")
date_tests = [
    DateTestCase("Added on Sunday, January 7, 2024 10:25:48 AM", "2024-01-07"),
    DateTestCase("Added on Wednesday, January 10, 2024 1:20:25 PM", "2024-01-10"),
    DateTestCase("Added on Wednesday, January 10, 2024 1:20:25 PM", "2024-01-10"),
    DateTestCase("Added on Monday, January 8, 2024 1:20:25 PM", "2024-01-08"),
    DateTestCase("Añadido el lunes, 10 de julio de 2023 13:23:06", "2023-07-10"),
    DateTestCase("", None),
]

NoteTypeTestCase = namedtuple("TestCase", "input note_type")
note_type_tests = [
    NoteTypeTestCase("Your Highlight on page 31", "highlight"),
    NoteTypeTestCase("La subrayado en la página 236", "highlight"),
    NoteTypeTestCase("", None),
]



import hashlib
from typing import Any, Dict

from my_clippings.utils.parsers import parse_metadata, parse_title_and_author


class Clipping:
    def __init__(self, raw_clipping: Dict[str, str]):
        self.raw = raw_clipping
        self.parsed = raw_clipping

    @property
    def raw(self) -> Dict[str, Any]:
        return self._raw

    @raw.setter
    def raw(self, raw_clipping):
        """Validate raw clipping"""
        required_keys = ["title_and_author", "metadata", "content"]
        for key in required_keys:
            if not raw_clipping[key]:
                raise ValueError(
                    f"{key} missing or empty in raw clipping: {raw_clipping}"
                )
        self._raw = raw_clipping

    @property
    def parsed(self) -> Dict[str, Any]:
        return self._parsed

    @parsed.setter
    def parsed(self, raw_clipping) -> Dict[str, Any]:
        """Parse raw clipping into a dict"""
        title_and_author = raw_clipping.get("title_and_author")
        metadata = raw_clipping.get("metadata")
        content = raw_clipping.get("content")

        if not title_and_author or not metadata or not content:
            raise ValueError(f"Invalid raw clipping: {raw_clipping}")

        id = hashlib.sha256(content.encode("utf8")).hexdigest()[:8]
        title, author = parse_title_and_author(title_and_author)
        note_type, location, date = parse_metadata(metadata)

        self._parsed = {
            "id": id,
            "title": title,
            "author": author,
            "note_type": note_type,
            "location": location,
            "date": date,
            "content": content,
        }
